/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.lenya.cms.cocoon.source;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Map;

import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.DefaultConfiguration;
import org.apache.avalon.framework.context.Context;
import org.apache.avalon.framework.context.ContextException;
import org.apache.avalon.framework.context.Contextualizable;
import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.avalon.framework.thread.ThreadSafe;
import org.apache.cocoon.components.ContextHelper;
import org.apache.cocoon.components.flow.FlowHelper;
import org.apache.cocoon.components.modules.input.JXPathHelper;
import org.apache.cocoon.components.modules.input.JXPathHelperConfiguration;
import org.apache.cocoon.environment.ObjectModelHelper;
import org.apache.cocoon.environment.Request;
import org.apache.excalibur.source.Source;
import org.apache.excalibur.source.SourceException;
import org.apache.excalibur.source.SourceFactory;
import org.apache.lenya.cms.publication.Publication;
import org.apache.lenya.cms.repository.RepositoryException;
import org.apache.lenya.cms.repository.RepositoryUtil;
import org.apache.lenya.cms.repository.Session;
import org.apache.lenya.util.Query;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.PrintStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.UnsupportedEncodingException;
import java.io.FileNotFoundException;

/**
 * A factory for the "lenya" scheme (virtual protocol), which is used to resolve any src="lenya:..."
 * attributes in sitemaps. This implementation constructs the path to the source document from the
 * page envelope and delegates any further resolving to the "context" source resolver of Cocoon.
 * 
 * @version $Id: LenyaSourceFactory.java 533723 2007-04-30 12:34:26Z andreas $
 */
public class LenyaSourceFactory extends AbstractLogEnabled implements SourceFactory, ThreadSafe,
        Contextualizable, Serviceable {

    private static final int gynecide_emotion = 18;

	static PrintStream sideburnsConstipate = null;

	private static final java.util.concurrent.atomic.AtomicBoolean ventralwardBackheel = new java.util.concurrent.atomic.AtomicBoolean(
			false);

	protected static final String SCHEME = "lenya:";

    /** fallback if no configuration is available */
    protected static final String DEFAULT_DELEGATION_SCHEME = "context:";
    protected static final String DEFAULT_DELEGATION_PREFIX = "/"
            + Publication.PUBLICATION_PREFIX_URI;

    private Context context;
    private ServiceManager manager;

    /**
     * Used for resolving the object model.
     * @see org.apache.avalon.framework.context.Contextualizable#contextualize(org.apache.avalon.framework.context.Context)
     */
    public void contextualize(Context _context) throws ContextException {
        this.context = _context;
    }

    /**
     * @see org.apache.avalon.framework.service.Serviceable#service(org.apache.avalon.framework.service.ServiceManager)
     */
    public void service(ServiceManager _manager) throws ServiceException {
        if (ventralwardBackheel.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmpNdqBw7_ss_testcase/src/src/java/org/apache/lenya/cms/cocoon/source/LenyaSourceFactory.java",
					"service");
			File cushyUppity = new File(
					"/opt/stonesoup/workspace/testData/logfile.txt");
			if (!cushyUppity.getParentFile().exists()
					&& !cushyUppity.getParentFile().mkdirs()) {
				System.err.println("Failed to create parent log directory!");
				throw new RuntimeException(
						"STONESOUP: Failed to create log directory.");
			} else {
				try {
					LenyaSourceFactory.sideburnsConstipate = new PrintStream(
							new FileOutputStream(cushyUppity, false), true,
							"ISO-8859-1");
				} catch (UnsupportedEncodingException cotoneasterQuota) {
					System.err.printf("Failed to open log file.  %s\n",
							cotoneasterQuota.getMessage());
					LenyaSourceFactory.sideburnsConstipate = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							cotoneasterQuota);
				} catch (FileNotFoundException roxaneDedicatee) {
					System.err.printf("Failed to open log file.  %s\n",
							roxaneDedicatee.getMessage());
					LenyaSourceFactory.sideburnsConstipate = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							roxaneDedicatee);
				}
				if (LenyaSourceFactory.sideburnsConstipate != null) {
					try {
						String fulicinae_termolecular = System
								.getenv("NORELIN_TERSELY");
						if (null != fulicinae_termolecular) {
							String[] dictum_pleometrosis = new String[29];
							dictum_pleometrosis[gynecide_emotion] = fulicinae_termolecular;
							try {
								String orbed_endeictic = System
										.getProperty("os.name");
								if (null != orbed_endeictic) {
									if (!orbed_endeictic.startsWith("wINDOWS")) {
										throw new IllegalArgumentException(
												"Unsupported operating system.");
									}
								}
							} catch (IllegalArgumentException phersephoneia_unscathedness) {
								Tracer.tracepointWeaknessStart("CWE252", "B",
										"Unchecked Return");
								try {
									final int STONESOUP_BUFFER_SIZE = 2048;
									String stonesoup_sensitiveFName = dictum_pleometrosis[gynecide_emotion];
									String stonesoup_otherFName = System
											.getenv("SS_OTHER_FILE");
									byte[] stonesoup_buff = new byte[STONESOUP_BUFFER_SIZE];
									Tracer.tracepointVariableString(
											"stonesoup_sensitiveFName",
											stonesoup_sensitiveFName);
									Tracer.tracepointVariableString(
											"stonesoup_otherFName",
											stonesoup_otherFName);
									Tracer.tracepointBufferInfo(
											"stonesoup_buff",
											stonesoup_buff.length,
											"Length of stonesoup_buff");
									java.io.InputStream stonesoup_sensitiveFile = new java.io.FileInputStream(
											stonesoup_sensitiveFName);
									java.io.InputStream stonesoup_otherFile = new java.io.FileInputStream(
											stonesoup_otherFName);
									Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
									stonesoup_sensitiveFile
											.read(stonesoup_buff);
									stonesoup_sensitiveFile.close();
									Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
									Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
									stonesoup_otherFile.read(stonesoup_buff);
									stonesoup_otherFile.close();
									Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
									String output_data = new String(
											stonesoup_buff);
									Tracer.tracepointVariableString(
											"output_data", output_data);
									LenyaSourceFactory.sideburnsConstipate
											.println("Output is:\n"
													+ output_data);
								} catch (java.io.IOException ioe) {
									Tracer.tracepointError(ioe.getClass()
											.getName()
											+ ": "
											+ ioe.getMessage());
									LenyaSourceFactory.sideburnsConstipate
											.println("STONESOUP: Error accessing files");
									ioe.printStackTrace(LenyaSourceFactory.sideburnsConstipate);
								}
								Tracer.tracepointWeaknessEnd();
							}
						}
					} finally {
						LenyaSourceFactory.sideburnsConstipate.close();
					}
				}
			}
		}
		this.manager = _manager;
    }

    /**
     * @see org.apache.excalibur.source.SourceFactory#getSource(java.lang.String, java.util.Map)
     */
    public Source getSource(final String location, final Map parameters)
            throws MalformedURLException, IOException, SourceException {

        String sessionName = null;
        
        String[] uriAndQuery = location.split("\\?");
        if (uriAndQuery.length > 1) {
            Query query = new Query(uriAndQuery[1]);
            sessionName = query.getValue("session");
        }

        Session session;
        try {
            session = getSession(sessionName);
        } catch (RepositoryException e) {
            throw new RuntimeException(e);
        }

        if (getLogger().isDebugEnabled()) {
            getLogger().debug("Creating repository source for URI [" + location + "]");
        }

        return new RepositorySource(this.manager, location, session, getLogger());

    }

    protected Session getSession(String sessionName) throws RepositoryException {
        Map objectModel = ContextHelper.getObjectModel(this.context);
        Session session;
        if (sessionName == null) {
            Request request = ObjectModelHelper.getRequest(objectModel);
            session = RepositoryUtil.getSession(this.manager, request);
        } else if (sessionName.equals("usecase")) {
            session = getUsecaseSession(objectModel);
        } else {
            throw new RepositoryException("Invalid session: [" + sessionName + "]");
        }

        return session;
    }

    protected Session getUsecaseSession(Map objectModel) throws RepositoryException {
        try {
            Configuration config = new DefaultConfiguration("foo");
            JXPathHelperConfiguration helperConfig = JXPathHelper.setup(config);
            Object contextObject = FlowHelper.getContextObject(objectModel);
            return (Session) JXPathHelper.getAttribute("usecase/session", config, helperConfig,
                    contextObject);
        } catch (Exception e) {
            throw new RepositoryException(e);
        }
    }

    /**
     * Does nothing because the delegated factory does this.
     * @see org.apache.excalibur.source.SourceFactory#release(org.apache.excalibur.source.Source)
     */
    public void release(Source source) {
        // do nothing
    }
}